﻿-- Author      : RisM
-- Create Date : 6/28/2009 4:03:23 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L			= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)
local HELPFILE	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell_HELPFILE",			false)

SpeakinSpell:PrintLoading("slashcommands.lua")

-------------------------------------------------------------------------------
-- SLASH COMMAND HANDLERS
-------------------------------------------------------------------------------

-- table of supporting slash commands
-- NOTE: support both English and current locale
--	non-English game clients can typically use both translated and untranslated slash commands
--	for built-in functions like /bg chat, etc, so it makes sense here as well
local SLASH_COMMANDS_SIMPLE = {
	["toggle"]		= function() SpeakinSpell:ShowOptions_Toggle() end,
	["options"]		= function() SpeakinSpell:ShowOptions() end,
	["create"]		= function() SpeakinSpell:ShowCreateNew() end,
	["messages"]	= function() SpeakinSpell:ShowMessageOptions() end,
	["random"]		= function() SpeakinSpell:ShowRandomSubsOptions() end,
	["import"]		= function() SpeakinSpell:ShowImport() end,
	["colors"]		= function() SpeakinSpell:ShowColorsGUI() end,
	["network"]		= function() SpeakinSpell:ShowNetworkOptions() end,
	["help"]		= function() SpeakinSpell:ShowHelp() end,
	["reset"]		= function() SpeakinSpell:ResetToDefaults(nil) end,
	["advertise"]	= function() SpeakinSpell:Advertise("") end,
	["ad"]			= function() SpeakinSpell:Advertise("") end,
	["ad /s"]		= function() SpeakinSpell:Advertise("SAY") end,
	["ad /p"]		= function() SpeakinSpell:Advertise("PARTY") end,
	["ad /ra"]		= function() SpeakinSpell:Advertise("RAID") end,
	["testallsubs"] = function() SpeakinSpell:TestAllSubs() end,
	["memory"]		= function() SpeakinSpell:Print( SpeakinSpell:GetAddonMemoryUsedString() ) end,
	["sync"]		= function() SpeakinSpell:Network_AutoSync() end,
	["guides"]		= function() SpeakinSpell:ToggleSetupGuides() end,
	["recent"]		= function() SpeakinSpell:ReportRecentEvents() end,
}

--NOTE: always include a space in the key, so that params will not start with that
local SLASH_COMMANDS_STARTSWITH = {
	["ad "]		= function( key, params ) SpeakinSpell:Advertise("WHISPER", params) end,
	["sync "]	= function( key, params ) SpeakinSpell:Network_SyncWithTarget( params ) end,
	["macro "]	= function( key, params ) SpeakinSpell:ProcessUserMacro( key, params ) end,
}


-------------------------------------------------------------------------------
-- SETUP
-------------------------------------------------------------------------------


function SpeakinSpell:RegisterChatCommands()
	-- add the localized slash commands (support both)
	-- NOTE: these are the starter-word only "/SpeakinSpell" and variants
	for _,command in pairs(SpeakinSpell.SlashCommands) do
		--local lkey = "/"..command -- no, but this is how it is for the keys to the next parameter in the input
		local lkey = command -- for this command prefix the Locale table specifies keys matching values in English
		local lcommand = L[lkey] -- returns lkey if not defined in Locale file
		if lcommand ~= lkey then
			tinsert(SpeakinSpell.SlashCommands, lcommand)
		end
	end
	
	-- register all chat commands
	for k,v in pairs(SpeakinSpell.SlashCommands) do
		self:RegisterChatCommand(v,"OnSlashCommand")
	end

	-- now for our own commands, which are technically parameters to the commands we just registered above
	
	-- add localized commands
	for command,Func in pairs(SLASH_COMMANDS_SIMPLE) do
		local lcommand = L[command]
		if lcommand ~= command and (not SLASH_COMMANDS_SIMPLE[ lcommand ]) then
			SLASH_COMMANDS_SIMPLE[ lcommand ] = Func
		end
	end
	-- and same thing for the Starts-With table
	for command,Func in pairs(SLASH_COMMANDS_STARTSWITH) do
		local lcommand = L[command]
		if lcommand ~= command and (not SLASH_COMMANDS_STARTSWITH[ lcommand ]) then
			SLASH_COMMANDS_STARTSWITH[ lcommand ] = Func
		end
	end
end



-------------------------------------------------------------------------------
-- OnSlashCommand main handler
-------------------------------------------------------------------------------


function SpeakinSpell:OnSlashCommand(input)
	local funcname = "OnSlashCommand"

	-- format <substutitions> in the input and lowercase it for table lookups
	local de = {
		name = input,
		target = UnitName("target"),
	}
	input = self:FormatSubs( string.lower(input), de )
	self:DebugMsg(funcname,"OnSlashCommand:"..tostring(input))
	
	-- if nothing is specified, then choose a default command
	if input == nil or input == "" then
		input = "toggle"
	end
	
	-- what did the user enter?
	Func = SLASH_COMMANDS_SIMPLE[input]
	
	-- process the command if found in the easy table
	if Func then
		Func()
		return
	end
	
	-- It wasn't an easy command...
	--	check for commands that start with one of the commands that have arguments
	--	defined in SLASH_COMMANDS_STARTSWITH[]

	for key,Func in pairs(SLASH_COMMANDS_STARTSWITH) do
		if self:StartsWith(input, key) then
			local params = string.sub( input, string.len(key)+1 )
			Func( key, params )
			return
		end
	end
	
	--else -- doesn't appear to be a valid command
	self:ShowHelp()
		
	-- go straight to the slash commands page (if possible)
	-- NOTE: this currently doesn't work... see notes in HelpPages_SelectTopic()
	self:HelpPages_SelectTopic( HELPFILE.SLASH_COMMANDS )

end


-------------------------------------------------------------------------------
-- /ss macro stuff
-------------------------------------------------------------------------------


function SpeakinSpell:ProcessUserMacro( key, params )
	--NOTE: legacy behavior here... TODOFUTURE: set name = only the params instead of the whole input
	local input = key..params -- "macro something" not including /ss
	
	self:DebugMsg(funcname, "User Macro input:"..tostring(params))
	
	local DetectedEventStub = {
		-- event descriptors
		name = input, -- use the entire lowercase input = "macro something"
		type = "MACRO",
		-- event-specific data for substitutions
		caster = UnitName("player"),
	}
	
	-- NOTE: do not inherit parent event data here
	--		that would corrupt the event key,
	--		causing us to load the wrong speech list and options
	--		and leading to infinite recursion
	--		see SpeakinSpell:InheritParentEvent()
	
	-- signal the event
	self:OnSpeechEvent( DetectedEventStub )
	return true
end


function SpeakinSpell:GetETEForSSMacro(input)
	local funcname = "GetETEForSSMacro"
	
	local macro = input
	for _,slash in pairs(SpeakinSpell.SlashCommands) do
		if self:StartsWith(macro, slash) then
			macro = string.sub(macro, string.len(slash)+2)
			break
		end
	end
	while self:StartsWith(macro, " ") do
		macro = string.sub(macro, 2)
	end
	if not ( self:StartsWith(macro, "macro ") or self:StartsWith(macro, L["/macro "]) ) then
		-- not a recursive macro
		--self:DebugMsg(funcname, string.format([["%s" doesn't start with "macro"]], macro) )
		return nil
	end
	
	local key = self:Keyify( "MACRO"..tostring(macro) )
	--self:DebugMsg(funcname, "key:"..key)
	return self:GetActiveEventTable()[key]
end


function SpeakinSpell:ProcessRecursiveUserMacro(input, ParentDetectedEvent)
	local funcname = "ProcessRecursiveUserMacro"

	local ete = self:GetETEForSSMacro(input)
	if not ete then
		-- this is not an error, because we call this function as a test
		-- but it does mean we have nothing to do here
		--self:ErrorMsg(funcname, "no settings defined for "..tostring(input))
		return false
	end
		
	if self.RuntimeData.RecursiveCall == input then
		self:ErrorMsg(funcname, "stopped infinite recursive call for "..tostring(input))
		return true --consider this processed and stop
	end
	
	self.RuntimeData.RecursiveCall = input
	self:DebugMsg(funcname, "Recursive User Macro input: "..tostring(self.RuntimeData.RecursiveCall))
	
	local DetectedEventStub = {
		-- event descriptors
		name = ete.DetectedEvent.name, -- use the entire lowercase input = "macro something"
		type = "MACRO",
		-- event-specific data for substitutions
		target = ParentDetectedEvent.target,
		caster = ParentDetectedEvent.caster,
	}
	
	-- signal the event
	self:OnRecursiveEventDetected( input, DetectedEventStub, ParentDetectedEvent )
	
	-- clear the infinite recursive call blocker
	self.RuntimeData.RecursiveCall = nil
	
	return true
end

